//
//  FactualFieldMetadata.h
//  FactualSDK
//
//  Version 1.0
//  Copyright 2011 Factual Inc. All rights reserved.
//

#import <Foundation/Foundation.h>


/*!@abstract Metadata associated with a single field in a Factual Table 
 @discussion
 
 */
@interface FactualFieldMetadata : NSObject
  
/*! @property 
 @discussion The name of the field
 */ 
@property (nonatomic,readonly) NSString* fieldName;
/*! @property 
 @discussion The numerical field id of the field
 */ 
@property (nonatomic,readonly) NSInteger fieldId;
/*! @property 
 @discussion The data type associated with this field
 */ 
@property (nonatomic,readonly) NSString* dataType;
/*! @property 
 @discussion a boolean set to True if this is a system generated field
 */ 
@property (nonatomic,readonly) BOOL isAutoGenerated;
/*! @property 
 @discussion a boolean set to True if this is a hidden  field
 */ 
@property (nonatomic,readonly) BOOL isHidden;
/*! @property 
 @discussion a boolean set to True if this is part of the primary key.
 */ 
@property (nonatomic,readonly) BOOL isPrimary;
/*! @property 
 @discussion a boolean set to True if this field's value can be changed via user
 feedback (votes/updates).
 */ 
@property (nonatomic,readonly) BOOL isVotable;
/*! @property 
 @discussion a boolean set to True if this an enumerated value field
 */ 
@property (nonatomic,readonly) BOOL isEnumerated;
/*! @property 
 @discussion an array of enumerated value strings if this field is an enumerated
 value field.
 */ 
@property (nonatomic,readonly) NSArray*  enumerationList;


@end
